@@ -1,6 +1,6 @@
 /*****************************************************************************
  *                                                                           *
- *          UNURAN -- Universal Non-Uniform Random number generator          *
+ *          UNU.RAN -- Universal Non-Uniform Random number generator         *
  *                                                                           *
  *****************************************************************************
  *                                                                           *
@@ -20,10 +20,11 @@
  *      pointer to the PDF, center of distribution                           *
  *                                                                           *
  *   OPTIONAL:                                                               *
+ *      pointer to CDF, pointer to derivative of PDF                         *
  *                                                                           *
  *****************************************************************************
  *                                                                           *
- *   Copyright (c) 2008 Wolfgang Hoermann and Josef Leydold                  *
+ *   Copyright (c) 2008-2010 Wolfgang Hoermann and Josef Leydold             *
  *   Department of Statistics and Mathematics, WU Wien, Austria              *
  *                                                                           *
  *   This program is free software; you can redistribute it and/or modify    *
@@ -45,6 +46,18 @@
  *                                                                           *
  *   REFERENCES:                                                             *
  *                                                                           *
+ *   [1] Derflinger, Gerhard, H{\"o}rmann, Wolfgang, and Leydold, Josef:     *
+ *       Random Variate Generation by Numerical Inversion when only the      *
+ *       Density Is Known. ACM Trans. Model. Comput. Simul. (2010),          *
+ *       to appear.                                                          *
+ *       See also Research Report Series of the Department of                *
+ *       Statistics and Mathematics at WU Vienna, No. 90, June 2009.         *
+ *       http://epub.wu.ac.at/, oai:epub.wu-wien.ac.at:epub-wu-01_f41        *
+ *                                                                           *
+ *   [2] H{\"o}rmann, Wolfgang and Leydold, Josef:                           *
+ *       Continuous Random Variate Generation by Fast Numerical Inversion,   *
+ *       ACM Trans. Model. Comput. Simul. 13(4), pp. 347--362, 2003.         *
+ *                                                                           *
  *****************************************************************************
  *                                                                           *
  *   Method PINV combines numerical integration with interpolation of the    *
@@ -65,6 +78,7 @@
  *         interpolating polynomial).                                        *
  *            _unur_pinv_computational_domain()                              *
  *            _unur_pinv_cut()                                               *
+ *            _unur_pinv_cut_bisect()                                        *
  *            _unur_pinv_cut_CDF()                                           *
  *                                                                           *
  *   1d.   Compute area below PDF over relevant domain with requested        *
@@ -78,6 +92,7 @@
  *         fixed (sub-) interval.                                            *
  *            _unur_pinv_newton_create()                                     *
  *            _unur_pinv_chebyshev_points()                                  *
+ *            _unur_pinv_newton_cpoints()                                    *
  *                                                                           *
  *   2b.   Evaluate interpolating polynomial.                                *
  *            _unur_pinv_newton_eval()                                       *
@@ -85,6 +100,7 @@
  *   2c.   Estimate approximation error for given interpolating polynomial.  *
  *            _unur_pinv_newton_maxerror()                                   *
  *            _unur_pinv_newton_testpoints()                                 *
+ *            _unur_pinv_cubic_hermite_monotone()                            *
  *                                                                           *
  *                                                                           *
  *   Currently the following methods are implemented:                        *
@@ -97,10 +113,18 @@
  *     Newton recursion for coefficients of polynomial                       *
  *     ("Newton interpolation").                                             *
  *                                                                           *
+ *     Hermite interpolation [2] (as the limit case with double and          *
+ *     tripple nodes).                                                       *
+ *                                                                           *
  *****************************************************************************/
 
 /*---------------------------------------------------------------------------*/
 
+/* enable additional code for development */
+/* #define PINV_DEVEL */
+
+/*---------------------------------------------------------------------------*/
+
 #include <unur_source.h>
 #include <distr/distr.h>
 #include <distr/distr_source.h>
@@ -118,7 +142,7 @@
 /* Constants                                                                 */
 
 /* -- Global parameters                                                      */
-#define MAX_ORDER   (12)
+#define MAX_ORDER   (17)
 /* Maximum order of Newton interpolation polynomial */
 
 #define PINV_UERROR_CORRECTION  (0.9)
@@ -184,8 +208,10 @@
 /*---------------------------------------------------------------------------*/
 /* Variants                                                                  */
 
-#define PINV_VARIANT_PDF      0x0010u   /* use PDF and Lobatto integration   */
-#define PINV_VARIANT_CDF      0x0020u   /* use CDF                           */
+#define PINV_VARIANT_PDF      0x0010u   /* use PDF and Lobatto integration
+	                                   [ if not present, use CDF ]       */
+#define PINV_VARIANT_UPOINTS  0x0040u   /* use Chebyshev points in u scale   */
+#define PINV_VARIANT_KEEPCDF  0x0080u   /* keep table for integration        */
 
 /*---------------------------------------------------------------------------*/
 /* Debugging flags                                                           */
@@ -202,12 +228,17 @@
 /*---------------------------------------------------------------------------*/
 /* Flags for logging set calls                                               */
 
-#define PINV_SET_ORDER          0x001u  /* order of polynomial               */
-#define PINV_SET_U_RESOLUTION   0x002u  /* maximal error in u                */
-#define PINV_SET_BOUNDARY       0x008u  /* boundary of computational region  */
-#define PINV_SET_SEARCHBOUNDARY 0x010u  /* search for boundary               */
-#define PINV_SET_VARIANT        0x020u  /* variant of algorithm              */
-#define PINV_SET_MAX_IVS        0x040u  /* maximum number of subintervals    */
+#define PINV_SET_ORDER          0x0001u  /* order of polynomial              */
+#define PINV_SET_ORDER_COR      0x1000u  /* ... corrected                    */
+#define PINV_SET_SMOOTH         0x0002u  /* smoothness of interpolant        */
+#define PINV_SET_SMOOTH_COR     0x2000u  /* ... corrected                    */
+#define PINV_SET_U_RESOLUTION   0x0004u  /* maximal error in u               */
+#define PINV_SET_UPOINTS        0x0008u  /* use Chebyshev points in u scale  */
+#define PINV_SET_BOUNDARY       0x0010u  /* boundary of computational region */
+#define PINV_SET_SEARCHBOUNDARY 0x0020u  /* search for boundary              */
+#define PINV_SET_VARIANT        0x0040u  /* variant of algorithm             */
+#define PINV_SET_MAX_IVS        0x0080u  /* maximum number of subintervals   */
+#define PINV_SET_KEEPCDF        0x0100u  /* keep table for integration       */
 
 /*---------------------------------------------------------------------------*/
 
@@ -325,11 +356,16 @@ static int _unur_pinv_computational_domain (struct unur_gen *gen);
 /*     interpolating polynomial).                                            */
 /*---------------------------------------------------------------------------*/
 
-static double _unur_pinv_cut (struct unur_gen *gen, double dom, double w, double dw, double crit);
+static double _unur_pinv_cut (struct unur_gen *gen, double w, double dw, double crit);
 /*---------------------------------------------------------------------------*/
 /* [1c.] calculate cut-off points for computational domain of distribution.  */
 /*---------------------------------------------------------------------------*/
 
+static double _unur_pinv_cut_bisect (struct unur_gen *gen, double x0, double x1);
+/*---------------------------------------------------------------------------*/
+/* [1c.] calculate cut-off points as boudary of bounded support of PDF.      */
+/*---------------------------------------------------------------------------*/
+
 static int _unur_pinv_computational_domain_CDF (struct unur_gen *gen);
 /*---------------------------------------------------------------------------*/
 /* 1c. Compute computational domain where inverse CDF is approximated        */
@@ -360,14 +396,15 @@ static int _unur_pinv_create_table( struct unur_gen *gen );
 /* create table for Newton interpolation                                     */
 /*---------------------------------------------------------------------------*/
 
-static int _unur_pinv_interval( struct unur_gen *gen, int i, double x, double cdfx );
+static int _unur_pinv_chebyshev_points (double *pt, int order, int smooth);
 /*---------------------------------------------------------------------------*/
-/* make a new interval i with left boundary point x and CDF(x).              */
+/* [2a.] Compute Chebyshev points.                                           */
 /*---------------------------------------------------------------------------*/
 
-static int _unur_pinv_lastinterval( struct unur_gen *gen );
+static int _unur_pinv_newton_cpoints (double *xval, int order, struct unur_pinv_interval *iv, 
+				      double h, double *chebyshev, int smooth, int use_upoints);
 /*---------------------------------------------------------------------------*/
-/* update size of array and set all uninitialized values to 0 that.          */
+/* [2a.] Compute points for construct interpolating polynomial.              */
 /*---------------------------------------------------------------------------*/
 
 static int _unur_pinv_newton_create (struct unur_gen *gen, struct unur_pinv_interval *iv, 
@@ -382,32 +419,42 @@ static int _unur_pinv_linear_create (struct unur_gen *gen, struct unur_pinv_inte
 /* [2a.] Compute coefficients for linear interpolation.                      */
 /*---------------------------------------------------------------------------*/
 
-static int _unur_pinv_chebyshev_points (int order, double *pt);
-/*---------------------------------------------------------------------------*/
-/* [2a.] Compute Chebyshev points.                                           */
-/*---------------------------------------------------------------------------*/
-
-static double _unur_pinv_newton_eval (double q, double ui[], double zi[], int order);
+static double _unur_pinv_newton_eval (double q, double *ui, double *zi, int order);
 /*---------------------------------------------------------------------------*/
 /* 2b. evaluate Newton polynomial.                                           */
 /*---------------------------------------------------------------------------*/
 
 static double _unur_pinv_newton_maxerror (struct unur_gen *gen, struct unur_pinv_interval *iv, 
-					  double xval[]);
+					  double *xval, int use_linear);
 /*---------------------------------------------------------------------------*/
 /* 2c. estimate maximal error of Newton interpolation in subinterval         */
 /*---------------------------------------------------------------------------*/
 
-static int _unur_pinv_newton_testpoints (int g, double ui[], double utest[]);
+static int _unur_pinv_newton_testpoints (double *utest, double ui[], int order);
 /*---------------------------------------------------------------------------*/
 /* [2c.] calculate the local maxima of the interpolation polynomial          */
 /*---------------------------------------------------------------------------*/
 
-static int _unur_pinv_linear_testpoints (int g, double ui[], double utest[]);
+static int _unur_pinv_linear_testpoints (double *utest, double *ui, int order);
 /*---------------------------------------------------------------------------*/
 /* [2c.] create table of test points for linear interpolation                */
 /*---------------------------------------------------------------------------*/
 
+static int _unur_pinv_cubic_hermite_is_monotone();
+/*---------------------------------------------------------------------------*/
+/* [2c.] check monotonicity of cubic Hermite interpolation                   */
+/*---------------------------------------------------------------------------*/
+
+static int _unur_pinv_interval( struct unur_gen *gen, int i, double x, double cdfx );
+/*---------------------------------------------------------------------------*/
+/* make a new interval i with left boundary point x and CDF(x).              */
+/*---------------------------------------------------------------------------*/
+
+static int _unur_pinv_lastinterval( struct unur_gen *gen );
+/*---------------------------------------------------------------------------*/
+/* update size of array and set all uninitialized values to 0.               */
+/*---------------------------------------------------------------------------*/
+
 
 /*.......................*/
 /*  file: pinv_debug.ch  */
@@ -481,6 +528,7 @@ static void _unur_pinv_info( struct unur_gen *gen, int help );
 #define SAMPLE    gen->sample.cont      /* pointer to sampling routine       */
 
 #define PDF(x)  (_unur_pinv_eval_PDF((x),(gen)))      /* call to PDF         */
+#define dPDF(x) (_unur_cont_dPDF((x),(gen->distr)))   /* call to derivate of PDF */
 #define CDF(x)  (_unur_cont_CDF((x),(gen->distr)))    /* call to CDF         */
 
 /*---------------------------------------------------------------------------*/
